/***** gd.c ***************************************
 * Description: Program to quantify genetic
 *   diversity.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Wed Feb 17 13:24:02 2010
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include "eprintf.h"
#include "interface.h"
#include "genDiv.h"

void runAnalysis(Args *args, int fd);

int main(int argc, char *argv[]){
  Args *args;
  char *version;
  int fd;
  int i;

  version = "0.12";
  setprogname2("gd");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  if(args->numInputFiles){
    for(i=0;i<args->numInputFiles;i++){
      fd = open(args->inputFiles[i],O_RDONLY,0);
      runAnalysis(args,fd);
      close(fd);
    }
  }else{
    fd = 0;
    runAnalysis(args,fd);
  }
  free(args);
  free(progname());
  return 0;
}

void runAnalysis(Args *args, int fd){
  Sequence *seq;
  Alignment *al;
  long i;
  double s, sum, p, winPos;
  double *arr;

  seq = readFasta(fd);
  al = seq2al(seq);
  if(args->P){
    printPoly(al);
    return;
  }
  if(args->w){
    args->w = args->w < al->n ? args->w : al->n;
    if(args->W == -1)
      args->W = args->w / 2 + 1;
    if(args->s == 's')
      arr = winSs(args,al);
    else if(args->s == 't')
      arr = winTajima(args,al);
    else
      arr = winPi(args,al);
    winPos = (args->w-1.)/2. + 1;
    for(i=0;i<al->numWin;i++){
      if(args->s != 't'){
	if(al->winNumNuc[i] >= args->W)
	  printf("%g\t%.6f\n",winPos,arr[i]/al->winNumNuc[i]);
      }else{
	printf("%g\t%.6f\n",winPos,arr[i]);
      }
      winPos += args->S;
    }
  }else{
    if(args->s == 's'){
      sum = 0;
      for(i=1;i<al->m;i++)
	sum += 1./i;
      s = ss(al);
      printf("S:\t%g\tnumsites:\t%ld\tS/site:\t%.6f\ttheta_W/site:\t%.6f\n", s,al->numNuc,s/al->numNuc,s/al->numNuc/sum);
    }else if(args->s == 't'){
      printf("D_T:\t%.6f\n",tajima(al));
    }else{
      p = pi(al);
      printf("pi:\t%.6f\tnumsites:\t%ld\tpi/site:\t%.6f\ttheta_pi/site:\t%.6f\n",p,al->numNuc,p/al->numNuc,p/al->numNuc);
    }
  }
}
