/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "w:W:S:s:hPv";

  args = (Args *)emalloc(sizeof(Args));
  args->w = 0;
  args->W = -1;
  args->s = 0;
  args->S = 0;
  args->h = 0;
  args->e = 0;
  args->v = 0;
  args->P = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'w':                           /* window analysis */
      args->w = atoi(optarg);
      break;
    case 'W':                           /* minimum number of nucleotide positions */
      args->W = atoi(optarg);
      if(args->W < 1)
	args->W = -1;
      break;
    case 'S':                           /* window analysis */
      args->S = atoi(optarg);
      break;
    case 's':                           /* statistic other than pi */
      args->s = optarg[0];
      break;
    case 'P':                           /* print polymorphisms */
      args->P = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    default:
      printf("# ERROR[gd]: unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  if(args->s != 0 && args->s != 'p' && args->s != 's' && args->s != 't'){
    printf("# ERROR[gd]: unknown statistic: %c\n",args->s);
    args->e = 1;
  }
  if(args->w && !args->S){
    args->S = args->w / 10;
    if(args->S < 1)
      args->S = 1;
  }
  return args;
}


void printUsage(char *version){
  printf("gd version %s\n", version);
  printf("purpose: quantify genetic diversity\n");
  printf("usage: gd [options] [inputFile(s)]\n");
  printf("options:\n");
  printf("\t[-s <CHAR> compute statistic CHAR, where CHAR may be\n");
  printf("\t\tp pi (default)\n");
  printf("\t\ts segregating sites\n");
  printf("\t\tt Tajima's D]\n");
  printf("\t[-w <NUM> sliding window analysis with window of length NUM; default: global analysis]\n");
  printf("\t[-W <NUM> sliding window contains >= NUM columns of nucleotides; default: NUM=1/2*winLen]\n");
  printf("\t[-S <NUM> step sliding window by NUM nucleotides; default: NUM=winLen/10]\n");
  printf("\t[-P print polymorphic sites and exit]\n");
  printf("\t[-v print version and other program information and exit]\n");			     
  printf("\t[-h print this help message and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("********************************************************\n");
  printf("*                 gd, version %s                     *\n", version);
  printf("*          Quantify Genetic Diversity                  *\n");
  printf("*               Bernhard Haubold                       *\n");
  printf("* CONTACT                                              *\n");
  printf("* haubold@evolbio.mpg.de                               *\n");
  printf("*                                                      *\n");
  printf("* LICENSE                                              *\n");
  printf("* This software is distributed under the GNU General   *\n");
  printf("* Public License. You should have received a copy      *\n");
  printf("* of the licence together with this software. If       *\n");
  printf("* not, see http://www.gnu.org/licenses/.               *\n");
  printf("********************************************************\n");
  exit(0);
}
