/***** queue.c ************************************
 * Description: Implementation of a FIFO queue for
 *   sliding window analysis. See Robert Sedgewick 
 *   (1998). Algorithms in C. Third edition, p. 
 *   153ff.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Wed Feb 17 09:36:47 2010
 **************************************************/
#include <stdlib.h>
#include "eprintf.h"
#include "queue.h"

static Item **q;
static int n, head, tail;

void queueInit(int maxN){
  q = (Item **)emalloc((maxN+1)*sizeof(Item *));
  n = maxN + 1;
  head = n;
  tail = 0;
}

int queueEmpty(){
  return head % n == tail;
}

void queuePut(Item *item){
  q[tail++] = item;
  tail = tail % n;
}

Item *queueGet(){
  head = head % n;
  return q[head++];
}

void queueFree(){
  while(!queueEmpty())
    free(queueGet());
  free(q);
}
