/***** sequenceData.h *******************************************************
 * Description: Header file for sequence manipulation tasks.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Jun  6 10:34:25 2004.
 * 
 * This file is part of kr,
 *
 * kr is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * kr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with kr; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 *****************************************************************************/ 
#define SEQLINE 1000       /* maximal length of one line in FASTA file; hard bound */
#define SEQBUFFER 5000000  /* define the size of the sequence buffer */
#define DICSIZE 256
#define BORDER 'Z' 
/* #define BORDER '\255' */

/* basic sequence type representing >= 1 entry in FASTA file */
typedef struct sequence{
  char *seq;                /* the sequence */
  char *id;                 /* the sequence id */
  int numSeq;               /* number of sequences represented */
  int numQuery;             /* number of query sequences */
  int *borders;             /* position of last character of sequence in seq */
  char **headers;           /* FASTA header lines */
  int len;                  /* sequence length */
  int *freqTab;             /* frequency table */
  int numQueryNuc;          /* number of nucleotides in query sequence */
  int numSbjctNuc;          /* number of nucleotides in sbjct sequence */
  int numNuc;               /* number of nucleotides in sequence */
  int queryStart;
  int queryEnd;
  int effQueryNuc;          /* number of nucleotides in query that are the starting
			      * point for shustrings that are longer than expected
			      * by chance alone */
  int *sbjctId;             /* sequence id for each sbjct position */
} Sequence;


Sequence *revcomp(Sequence *seq);
Sequence *getNextSequence(FILE *fp);

int *getDnaDictionary(int *dic);
int *getRestrictedDnaDictionary(int *dic);

Sequence *freeSequence(Sequence *seq);
Sequence *getPermanentNextSequence(FILE *fp);
void convertToAcgt(Sequence *seq);
void resetSequenceReader();
Sequence *readFasta(int fd);
Sequence **sequence2array(Sequence *seq);
void prepareSeq(Sequence *sequence);
Sequence *catSeq(Sequence *seq1, Sequence *seq2);
void randomizeSeq(Sequence *seq);
void randomizeSbjct(Sequence *seq);
Sequence *cloneSeq(Sequence *ori);
double gcContent(Sequence *seq);
